/*
 * futf8open.c
 * Public domain
 * 2018   A. Kakuto
 *
 */

/*
  used only on windows with Visual Studio
*/
#ifdef _MSC_VER

#include <kpathsea/config.h>
#include <kpathsea/c-ctype.h>
#include <kpathsea/line.h>
#include <kpathsea/readable.h>
#include <kpathsea/variable.h>
#include <kpathsea/absolute.h>
#include <wchar.h>

/*
  Get wide string from multibyte string. (by T. Tanaka)
*/
static wchar_t *
get_wstring_from_mbstring(int cp, const char *mbstr, wchar_t *wstr)
{
  int len;

  len = MultiByteToWideChar(cp, 0, mbstr, -1, wstr, 0);
  if (len==0) {
    return NULL;
  }
  if (wstr==NULL) {
    wstr = malloc(sizeof(wchar_t)*(len+1));
  }
  len = MultiByteToWideChar(cp, 0, mbstr, -1, wstr, len+1);
  if (len==0) {
    return NULL;
  }
  return wstr;
}

extern int  command_line_enc;

FILE *futf8open(const char *filename, const char *mode)
{
  wchar_t *wfilename, *wmode;
  FILE *ret;
  int  cp;

  if (command_line_enc == 0)
    return fopen(filename, mode);

  cp = CP_UTF8;
  wfilename = get_wstring_from_mbstring(cp, filename, wfilename=NULL);
  if (wfilename == NULL)
    return NULL;
  wmode = get_wstring_from_mbstring(cp, mode, wmode=NULL);
  if (wmode == NULL)
    return NULL;
  ret = _wfopen((const wchar_t *)wfilename, (const wchar_t *)wmode);
  free(wfilename);
  free(wmode);
  return ret;
}

extern FILE *xfopen (const char *filename, const char *mode);

FILE *xfutf8open(const char *filename, const char *mode)
{
  wchar_t *wfilename, *wmode;
  FILE *ret;
  int  cp;

  if (command_line_enc == 0)
    return xfopen(filename, mode);

  cp = CP_UTF8;
  wfilename = get_wstring_from_mbstring(cp, filename, wfilename=NULL);
  if (wfilename == NULL) {
    perror (filename);
    exit (1);
  }
  wmode = get_wstring_from_mbstring(cp, mode, wmode=NULL);
  if (wmode == NULL) {
    perror (filename);
    exit (1);
  }
  ret = _wfopen((const wchar_t *)wfilename, (const wchar_t *)wmode);
  free(wfilename);
  free(wmode);
  if (ret == NULL) {
    perror (filename);
    exit (1);
  }
  return ret;
}

int utf8_stat(const char *path, struct stat *buffer)
{
  wchar_t *wpath;
  int     ret;

  if (command_line_enc == 0)
    return stat(path, buffer);

  wpath = get_wstring_from_mbstring(CP_UTF8, path, wpath = NULL);
  ret = _wstat(wpath, buffer);
  free(wpath);
  return ret;
}
#endif /* _MSC_VER */
